/*
 * ${license.title}
 * Copyright (C) 2022 ${license.owner}
 * ${license.mailto}
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#include "fggl/audio/openal/audio.hpp"

#include "fggl/util/service.hpp"
#include "fggl/data/storage.hpp"

namespace fggl::audio::openal {

	void AudioServiceOAL::play(const std::string &filename, bool looping) {

		// load audio clip into temp storage
		AudioClip clip;
		auto& locator = util::ServiceLocator::instance();
		auto storage = locator.get<data::Storage>();
		bool result = storage->load(data::StorageType::Data, filename, &clip);
		if ( !result ) {
			std::cerr << "error: can't load audio data" << std::endl;
		}

		play(clip, looping);
	}

	void AudioServiceOAL::play(AudioClip &clip, bool looping) {
		// play the audio on the default (non-positioned) source
		if ( m_defaultSource != nullptr ) {
			m_defaultSource->play(clip, looping);
		}
	}
}