//
// Created by webpigeon on 20/11/2021.
//

#include <fggl/data/model.hpp>
#include <fggl/data/heightmap.hpp>

// adapted from  https://www.mbsoftworks.sk/tutorials/opengl4/016-heightmap-pt1-random-terrain/

namespace fggl::data {

    void gridVertexNormals(data::Vertex *locations) {
        const int sizeX = data::heightMaxX;
        const int sizeY = data::heightMaxZ;
        const int gridOffset = sizeX * sizeY;

        // calculate normals for each triangle
        math::vec3* triNormals = new math::vec3[sizeX * sizeY * 2];
        for (int i = 0; i < sizeX - 1; i++) {
            for (int j = 0; j < sizeY - 1; j++) {
                // calculate vertex
                const auto &a = locations[i * sizeY + j].posititon;
                const auto &b = locations[(i + 1) * sizeY + j].posititon;
                const auto &c = locations[i * sizeY + (j + 1)].posititon;
                const auto &d = locations[(i + 1) * sizeY + (j + 1)].posititon;

                const auto normalA = glm::cross(b - a, a - d);
                const auto normalB = glm::cross(d - c, c - b);

                // store the normals
                int idx1 = idx(i, j, sizeY);
                int idx2 = idx1 + gridOffset;
                triNormals[idx1] = glm::normalize(normalA);
                triNormals[idx2] = glm::normalize(normalB);
            }
        }

        // calculate normals for each vertex
        for (int i = 0; i < sizeX; i++) {
            for (int j = 0; j < sizeY; j++) {
                const auto firstRow = (i == 0);
                const auto firstCol = (j == 0);
                const auto lastRow = (i == sizeX - 1);
                const auto lastCol = (i == sizeY - 1);

                auto finalNormal = glm::vec3(0.0f, 0.0f, 0.0f);

                if (!firstRow && !firstCol) {
                    finalNormal += triNormals[idx(i - 1, j - 1, sizeY)];
                }

                if (!lastRow && !lastCol) {
                    finalNormal += triNormals[idx(i, j, sizeY)];
                }

                if (!firstRow && lastCol) {
                    finalNormal += triNormals[idx(i - 1, j, sizeY)];
                    finalNormal += triNormals[idx(i - 1, j, sizeY) + gridOffset];
                }

                if (!lastRow && !firstCol) {
                    finalNormal += triNormals[idx(i, j - 1, sizeY)];
                    finalNormal += triNormals[idx(i, j - 1, sizeY) + gridOffset];
                }

                locations[idx(i, j, sizeY)].normal = glm::normalize(finalNormal) * -1.0f; //FIXME the normals seem wrong.
            }
        }
        delete[] triNormals;
    }

    void generateHeightMesh(const data::HeightMap *heights, data::Mesh &mesh) {

        // step 1: convert height data into vertex locations
        const int numElms = data::heightMaxX * data::heightMaxZ;
		std::array<data::Vertex, numElms> locations{};

		// iterate the
        for (std::size_t x = 0; x < data::heightMaxX; x++) {
            for (std::size_t z = 0; z < data::heightMaxZ; z++) {
                float level = heights->getValue(x, z);
                auto xPos = float(x);
                auto zPos = float(z);

                std::size_t idx1 = idx(x, z, data::heightMaxZ);
                locations[idx1].colour = fggl::math::vec3(1.0f, 1.0f, 1.0f);
                locations[idx1].posititon = math::vec3(-0.5f + xPos, level, -0.5f - zPos);
            }
        }
        gridVertexNormals(locations.data());

        mesh.restartVertex = data::heightMaxZ * data::heightMaxX;

        // populate mesh
        for (auto i = 0; i < numElms; i++) {
            mesh.pushVertex(locations[i]);
        }

        for (std::size_t x = 0; x < data::heightMaxX - 1; x++) {
            for (std::size_t z = 0; z < data::heightMaxZ; z++) {
                for (int k=0; k < 2; k++) {
                    auto idx = (x+k) * data::heightMaxZ + z;
                    mesh.pushIndex(idx);
                }
            }
            mesh.pushIndex(mesh.restartVertex);
        }

    }

}
