/*
 * ${license.title}
 * Copyright (C) 2022 ${license.owner}
 * ${license.mailto}
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

//
// Created by webpigeon on 23/04/22.
//

#include "fggl/scenes/game.hpp"
#include "fggl/util/service.hpp"
#include "fggl/phys/bullet/types.hpp"

namespace fggl::scenes {

	Game::Game(fggl::App &app) : AppState(app) {
		auto& locator = fggl::util::ServiceLocator::instance();
		m_input = locator.get<fggl::input::Input>();
	}

	void Game::activate() {
		fggl::AppState::activate();

		// setup the scene
		m_world = std::make_unique<ecs3::World>(*m_owner.registry());

		// FIXME this ties bullet to the game state - which shouldn't be the case
		m_phys = std::make_unique<phys::bullet::BulletPhysicsEngine>(m_world.get());
	}

	void Game::deactivate() {
		m_phys.reset();
		m_world.reset();
	}

	void Game::update() {
		if ( m_input != nullptr ) {
			bool escapePressed = m_input->keyboard.pressed(glfwGetKeyScancode(GLFW_KEY_ESCAPE));
			if (escapePressed) {
				m_owner.change_state(m_previous);
			}
		}

		if ( m_phys != nullptr ) {
			m_phys->step();
		}

		m_world->reapEntities();
	}

	void Game::render(fggl::gfx::Graphics &gfx) {
		if ( m_world != nullptr ) {
			gfx.drawScene( *m_world );
		}
	}

} // namespace demo

